﻿using Microsoft.Extensions.DependencyInjection;
using HealthMonitor.Core.Cache;
using HealthMonitor.Util.Common;
using HealthMonitor.Util.Entities.Interfaces;

namespace HealthMonitor.Core.Pipeline.Aop.Update
{
    public class UpdateEntityCacheValue : AopValueBase
    {
        public override async Task Invoke(CacheInterceptorContext context)
        {
            var entityType = context.EntityType;
            var param = context.AopContext.Parameters[0];

            //自增型主键无法预测其键值，所以不做缓存
            if (EntityUtils.HasAutoGenerateKey(entityType))
            {
                await InvokeNextAsync(context);
                return;
            }

            var durableManager = context.AopContext.ServiceProvider.GetService<IDurableEntityManager>();
            var cacheHandler = durableManager!.GetCacheHandler(entityType);
            //若实体类型不支持缓存，则退出
            if (cacheHandler == null || cacheHandler.DurableSecond <= 0)
            {
                await InvokeNextAsync(context);
                return;
            }

            string id = entityType.GetMethod(nameof(IEntity.GetPrimaryKey)).Invoke(param, null) + "";
            if (!string.IsNullOrEmpty(id))
            {
                string key = durableManager.CalcCacheKey(entityType, id);
                if (!string.IsNullOrEmpty(key))
                {
                    cacheHandler.SetEntityCache(key, param);
                }
            }

            await InvokeNextAsync(context);
        }
    }
}
